﻿using System;
using System.Collections.Generic;
using System.Linq;

using Avalonia.Threading;

namespace SddpViewer;

using System.Collections.ObjectModel;
using System.Threading.Tasks;

using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;

using Rssdp;

public partial class MainWindowViewModel : ObservableObject
{
    public MainWindowViewModel()
    {
        NetworkAdapters = NetworkAdapter.GetAvailableNetworkAdapter().ToArray();
        SelectedNetworkAdapter = NetworkAdapters.FirstOrDefault();
    }

    private async void LocatorOnDeviceUnavailable(object? sender, DeviceUnavailableEventArgs e) { }

    private async void LocatorOnDeviceAvailable(object? sender, DeviceAvailableEventArgs e)
    {
        if (e.IsNewlyDiscovered)
        {
            var discoveredDeviceViewModel = new DiscoveredDeviceViewModel(e.DiscoveredDevice);
            Dispatcher.UIThread.Invoke(() => SddpDevices.Add(discoveredDeviceViewModel));
            await discoveredDeviceViewModel.GetFurtherInformationAsync();
        }
    }

    [ObservableProperty]
    private IReadOnlyList<NetworkAdapter> _networkAdapters;

    [ObservableProperty]
    private NetworkAdapter? _selectedNetworkAdapter;

    [ObservableProperty]
    private string _deviceIpAddress = "192.168.42.193";

    [ObservableProperty]
    private string _notificationFilter = "upnp:rootdevice";

    private SsdpDeviceLocator? _locator;

    public ObservableCollection<DiscoveredDeviceViewModel> SddpDevices { get; } = new();

    [RelayCommand]
    private async Task SearchDevicesNowAsync()
    {
        SddpDevices.Clear();
        if (_locator is not null)
        {
            _locator.StopListeningForNotifications();
            _locator.DeviceAvailable -= LocatorOnDeviceAvailable;
            _locator.DeviceUnavailable -= LocatorOnDeviceUnavailable;
        }
        _locator = new SsdpDeviceLocator(SelectedNetworkAdapter?.IpAddress.ToString());
        if (!string.IsNullOrWhiteSpace(NotificationFilter))
        {
            _locator.NotificationFilter = NotificationFilter;
        }

        _locator.DeviceAvailable += LocatorOnDeviceAvailable;
        _locator.DeviceUnavailable += LocatorOnDeviceUnavailable;
        _locator.StartListeningForNotifications();
        await _locator.SearchAsync();
    }
}
